# This is part of colourettu. See http://minchin.ca/colourettu/

from __future__ import absolute_import

from PIL import Image

from ._colour import Colour, blend


class Palette:
    """Class for creating a palette of colours.

    A palette here is a list of colours.

    Args:
        start_colour (colourettu.Colour): the colour you want your palette to
            start with.
        end_colour (colourettu.Colour): the colour you want your palette to
            end with.

    .. note::

        If a *string*, *tuple*, or *list* is provided for `start_colour` or
        `end_colour`, a conversion to a :py:class:`colourettu.Colour` object
        will be attempted.

    .. code:: python

        from colourettu import Palette

        p1 = Palette()
        p1.to_image('p1.png', 60)

    .. image:: p1.png

    .. code:: python

        all_colours = [c1, c2, c3, c4, c5, c6]
        p2 = Palette()
        p2.from_list(all_colours)
        p2.to_image('p2.png', max_width=360, vertical=False)

    .. image:: p2.png

    """

    # TODO: Convert these code examples to an external file, so we can
    # auto-generate the .png files!
    #
    # see http://sphinx-doc.org/markup/code.html#includes

    _start = None
    _end = None
    _colours = []

    def __init__(self, start_colour=Colour("#FFF"), end_colour=Colour("#000")):
        # testing to see if the type is colourettu.colour throws an error.
        # This is an ugly hack.
        # TODO: Fix this!
        colour_for_type = Colour()
        if type(start_colour) is type(colour_for_type):
            self._start = start_colour
        else:
            try:
                self._start = Colour(start_colour)
            except:
                raise ValueError("Invalid start_colour given.")
        if type(end_colour) is type(colour_for_type):
            self._end = end_colour
        else:
            try:
                self._end = Colour(end_colour)
            except:
                raise ValueError("Invalid end_colour given.")

        self._colours = [self._start, self._end]

    def __repr__(self):
        return('<colourettu.Palette {} to {}, {} colours>'.format(
               self._start, self._end, len(self._colours)))

    def __str__(self):
        return('{}'.format(", ".join([c.hex() for c in self._colours])))

    def __len__(self):
        return(len(self._colours))

    def __add__(self, other):
        """
        Combine two Palettes.

        - adding two `colourettu.Palette`s will concatenate the two together
        - if two `colourettu.Palette`s are added, and the last colour of the
        first palette is the same as the first colour of the second palette,
        that colour will only appear once in the new palette
        - `colourettu.Palette`s and `colourettu.Colour`s can be added to
        create new `colourettu.Palette`s
        """
        colour_for_type = Colour()
        palette_for_type = Palette()
        if type(other) is type(colour_for_type):
            self._colours.append(other)
            self._end = other
            return self
        elif type(other) is type(palette_for_type):
            if (self._end == other._start):
                self._colours.extend(other._colours[1:])
            else:
                self._colours.extend(other._colours)
            self._end = other._end
            return self
        else:
            raise TypeError('unsupported opperand type(s) for +: {} and'
                            '{}'.format(type(self), type(other)))

    def __radd__(self, other):
        # used for `Colour + Palette`
        colour_for_type = Colour()
        if type(other) is type(colour_for_type):
            self._colours.insert(0, other)
            self._start = other
            return self
        else:
            raise TypeError('unsupported opperand type(s) for +: {} and'
                            '{}'.format(type(self), type(other)))

    # TO-DO     def __init__(self)   #this allows :  for colour in palette:

    def from_list(self, list_of_colours, normalized_rgb=False):
        """
        Given an interable (usually a list or a tuple) containing
        :py:class:`Colour` s,
        this then becomes the Colours contained by the Palette.

        Args:
            list_of_colours(list, tuple, or other interable): a collection of
                :py:class:`Colour` s to be loaded into the palette. If these
                are :py:class:`colourettu.Colour` 's, they will be loaded
                directly. Otherwise, an attempt to convert each item to a
                :py:class:`colourettu.Colour` will take place.
            normalized_rgb(bool): assuming the list is to be converted to
                :py:class:`colourettu.Colour` s, this parameter is passed on
                as part of
                that conversion process.

        .. note:

            This will overwrite the colours already defined by the palette.

        """
        colour_for_type = Colour()
        self._colours = []

        for c in list_of_colours:
            if type(c) is type(colour_for_type):
                self._colours.append(c)
            else:
                self._colours.append(Colour(c, normalized_rgb))

        self._start = self._colours[0]
        self._end = self._colours[-1]

    def to_image(self, filename='palette.png', band_width=1, length=60,
                 max_width=0, vertical=True, alpha_channel=False):
        """
        Creates an image from the palette.

        Args:
            filename(Optional[string]): filename of saved file. Defaults to
                ``palette.png`` in the current working directory.
            band_width(optional[int]): how wide each colour band should be.
                Defaults to 1 pixel.
            length(Optional[int]): the length of the overall image in pixels.
                This is the dimension orthogonal to ``band_width``. Defaults
                to 60 pixels.
            max_width(Optional[int]): if ``band_width`` is not set and this is,
                this determines how wide the whole image should be.
            vertical(Optional[bool]): if the image runs vertical (``True``,
                default) or horizontal (``False``).
            alpha_channel(Optional[bool]): if ``True``, the created image will
                have an Alpha channel. Defaults to ``False``.
        """
        # max_width is approximate
        # generate output pictures for documentation automatically

        if max_width < 1:
            pass
        else:
            band_width = int(max_width/len(self._colours))

        image_width = band_width * len(self._colours)

        if alpha_channel:
            my_image = Image.new('RGBA', (image_width, length))
        else:
            my_image = Image.new('RGB', (image_width, length))
        image_loaded = my_image.load()

        x = 0
        for my_colour in self._colours:
            for x1 in range(band_width):
                for y in range(length):
                    image_loaded[x, y] = my_colour.rgb()
                x = x + 1

        if vertical:
            my_image = my_image.rotate(270)

        my_image.save(filename)

    def blend(self, cycles=1):
        """
        Explands the existing Palette by inserting the blending colour
        between all Colours already in the Palette.

        Changes the Palette in-place.

        args:
            cycles(int): number of *blend* cycles to apply. (Default is 1)

        Example usage:

        .. code-block:: python

            p1.blend()
            p1.to_image('p1_blended.png', 60, vertical=False)

        .. image:: p1_blended.png

        .. code-block:: python

            p2.blend()
            p2.to_image('p2_blended.png', 60, vertical=False)

        .. image:: p2_blended.png

        The *blend* functionallity can be applied several times in a sequence
        by use of the *cycles* parameter. This may be useful to quickly get a
        longer series of intermediate colours.

        .. code-block:: python

            p3 = Palette(Colour('#fff'), Colour('#7e1e9c'))
            p3.blend(cycles=5)
            p3.to_image('p3.png', max_width=360, vertical=False)

        .. image:: p3.png

        .. seealso:: :py:func:`colourettu.blend`
        """

        for j in range(int(cycles)):
            new_colours = []
            for i, c in enumerate(self._colours):
                if i != 0:
                    c2 = blend(c, self._colours[i-1])
                    new_colours.append(c2)
                new_colours.append(c)

            self._colours = new_colours



"""
Colourettu is a collection of colour related functions.

Colourettu is a small collection of colour functions in Python. These can be
used to determine the (relative) luminosity of a colour and the contrast
between two colours. There is also the palette class for dealing with a 'list'
of colours.
"""

# This is part of colourettu. See http://minchin.ca/colourettu/

from __future__ import absolute_import


# METADATA

__title__ = "Colourettu"
__description__ = "Colourettu is a collection of colour related functions."
__url__ = "http://minchin.ca/colourettu/"
__author__ = "William Minchin"
__email__ = "w_minchin@hotmail.com"
__license__ = "MIT License"
__copyright_years__ = '2014-16'
__copyright__ = "Copyright (c) {} {}".format(__copyright_years__, __author__)

"""
This project uses the Semantic Versioning scheme in conjunction with PEP 0440:

    <http://semver.org/>
    <https://www.python.org/dev/peps/pep-0440>

Major versions introduce significant changes to the API, and backwards
compatibility is not guaranteed. Minor versions are for new features and other
backwards-compatible changes to the API. Patch versions are for bug fixes and
internal code changes that do not affect the API.

Version 0.x should be considered a development version with an unstable API,
and backwards compatibility is not guaranteed for minor versions.
"""
__version__ = "2.0.0"

# Package Implementation

from ._colour import *
from ._palette import *




import unittest
from unittest import expectedFailure, skip

import colourettu


class Test_Colour(unittest.TestCase):

    def test_default_colour(self):
        '''Default colour is white'''
        colour1 = colourettu.Colour()
        self.assertEqual(colour1._r, 255)
        self.assertEqual(colour1._g, 255)
        self.assertEqual(colour1._b, 255)

    def test_3hex_colour(self):
        '''Set the colour as like #dc8'''
        colour1 = colourettu.Colour('#dc8')
        self.assertEqual(colour1._r, 221)
        self.assertEqual(colour1._g, 204)
        self.assertEqual(colour1._b, 136)

    def test_6hex_colour(self):
        '''Set the colour as like #123456'''
        colour1 = colourettu.Colour('#123456')
        self.assertEqual(colour1._r, 18)
        self.assertEqual(colour1._g, 52)
        self.assertEqual(colour1._b, 86)

    def test_red(self):
        '''Get red of colour'''
        colour1 = colourettu.Colour('#D00')
        self.assertEqual(colour1.red(), 221)

    def test_green(self):
        '''Get green of colour'''
        colour1 = colourettu.Colour('#0B0')
        self.assertEqual(colour1.green(), 187)

    def test_blue(self):
        '''Get blue of colour'''
        colour1 = colourettu.Colour('#00A')
        self.assertEqual(colour1.blue(), 170)

    def test_rgb(self):
        '''Get rgb tuple of colour'''
        colour1 = colourettu.Colour('#345')
        self.assertEqual(colour1.rgb(), (51, 68, 85))

    def test_hex(self):
        '''Get hex value of colour'''
        colour1 = colourettu.Colour()
        self.assertEqual(colour1.hex(), "#FFFFFF")

    def test_hex_black(self):
        '''Get hex value of colour with leading zeros'''
        colour1 = colourettu.Colour("#0102DD")
        self.assertEqual(colour1.hex(), "#0102DD")

    def test_list(self):
        '''Get value of colour given as a list'''
        colour1 = colourettu.Colour([5, 10, 25])
        self.assertEqual(colour1.hex(), "#050A19")

    def test_tuple(self):
        '''Get value of colour given as a tuple'''
        colour1 = colourettu.Colour((35, 60, 128))
        self.assertEqual(colour1.hex(), "#233C80")

    def test_list_normalized_rgb(self):
        '''Get value of colour given as a list of normalized rgb values'''
        colour1 = colourettu.Colour([0.24287275804811442, 0.4339925778684171, 0.16562176715691224], normalized_rgb=True)
        self.assertEqual(colour1.hex(), "#3D6E2A")

    def test_tuple_normalized_rgb(self):
        '''Get value of colour given as a tuple of normalized rgb values'''
        colour1 = colourettu.Colour((0.5656023325553875, 0.8070789468680986, 0.8006291331865334), normalized_rgb=True)
        self.assertEqual(colour1.hex(), "#90CDCC")

    def test_tuple_normalized_rgb_value_too_big(self):
        '''Fail on invalid normalized RGB values (too big, tuple)'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour((1.2, 0.4, 0.378), normalized_rgb=True)

    def test_tuple_normalized_rgb_value_too_small(self):
        '''Fail on invalid normalized RGB values (too small, tuple)'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour((0.28, -0.4, 0.378), normalized_rgb=True)

    def test_list_normalized_rgb_value_too_big(self):
        '''Fail on invalid normalized RGB values (too big, list)'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour([0.289, 0.289, 1.33], normalized_rgb=True)

    def test_list_normalized_rgb_value_too_small(self):
        '''Fail on invalid normalized RGB values (too small, list)'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour([-0.289, 0.289, 0.33], normalized_rgb=True)

    def test_tuple_not_normalized(self):
        '''Fail if something other than `True` or `False` is passed for `normalized_rgb`'''
        with self.assertRaises(TypeError):
            colour1 = colourettu.Colour(normalized_rgb='stuff')

    @skip('Not yet defined')
    def test_list_not_normalized(self):
        pass

    @skip('Not yet defined')
    def test_tuple_for_normalized_rgb_too_long(self):
        pass

    @skip('Not yet defined')
    def test_tuple_for_normalized_rgb_too_short(self):
        pass

    def test_normalized_value_zero_float(self):
        '''Pass a value of 0 for a normalized RGB value'''
        colour1 = colourettu.Colour([0.0, 0.0, 0.0], normalized_rgb=True)
        colour2 = colourettu.Colour('#000')
        self.assertEqual(colour1, colour2)

    def test_normalized_value_one_float(self):
        '''Pass a value of 1 for a normalized RGB value'''
        colour1 = colourettu.Colour([1.0, 1.0, 1.0], normalized_rgb=True)
        colour2 = colourettu.Colour('#FFF')
        self.assertEqual(colour1, colour2)

    def test_normalized_value_zero_int(self):
        '''Pass a value of 0 for a normalized RGB value'''
        colour1 = colourettu.Colour([0, 0, 0], normalized_rgb=True)
        colour2 = colourettu.Colour('#000')
        self.assertEqual(colour1, colour2)

    def test_normalized_value_one_int(self):
        '''Pass a value of 1 for a normalized RGB value'''
        colour1 = colourettu.Colour([1, 1, 1], normalized_rgb=True)
        colour2 = colourettu.Colour('#FFF')
        self.assertEqual(colour1, colour2)

    @expectedFailure
    def test_colour_vs_color(self):
        '''test alternate spelling
        Removed in v1.0.0'''
        self.assertEqual(colourettu.Color, colourettu.Colour)

    def test_bad_hex_string_length(self):
        '''Invalid hex string should raise error'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour("#dddd")

    def test_not_hex_string(self):
        '''Stings must start with '#', otherwiser raise Error'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour("dddd")

    def test_bad_hex_chars(self):
        '''Hex strings that don't contain only hex characters [0-9a-f]
        should raise error'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour('#asdfgh')

    def test_bad_list_legnth(self):
        '''Lists must be 3 long, otherwise raise error'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour([1, 1, 1, 1])

    def test_bad_tuple_legnth(self):
        '''Tuples must be 3 long, otherwise raise error'''
        with self.assertRaises(ValueError):
            colour1 = colourettu.Colour((1, 1, 1, 1))

    def test_bad_list_value(self):
        '''Lists my contain integer, otherwise raise error'''
        with self.assertRaises(TypeError):
            colour1 = colourettu.Colour([1, 2, "stuff"])

    def test_bad_Tuple_value(self):
        '''Tuples my contain integer, otherwise raise error'''
        with self.assertRaises(TypeError):
            colour1 = colourettu.Colour((1, 2, "stuff"))

    def test_rgb_normalized_white(self):
        '''Get normalized rgb tuple of white'''
        colour1 = colourettu.Colour('#FFF')
        self.assertEqual(colour1.normalized_rgb(), (1, 1, 1))

    def test_rgb_nomralized_black(self):
        '''Get normalize rgb tuple of black'''
        colour1 = colourettu.Colour('#000')
        self.assertEqual(colour1.normalized_rgb(), (0, 0, 0))

    def test_repr(self):
        '''Representation of `Colour` class'''
        colour1 = colourettu.Colour()
        self.assertEqual(colour1.__repr__(), "<colourettu.Colour #FFFFFF>")

    def test_str(self):
        '''String representation of `Colour` class'''
        colour1 = colourettu.Colour()
        self.assertEqual(colour1.__str__(), "#FFFFFF")

    def test_eq(self):
        '''Equality operator'''
        c1 = colourettu.Colour()
        c2 = colourettu.Colour()
        c3 = colourettu.Colour('#abc')
        c4 = colourettu.Colour('#abc')
        c5 = colourettu.Colour('#123456')
        c6 = c5
        c7 = colourettu.Colour('#000000')
        c8 = colourettu.Colour([0, 0, 0])

        self.assertEqual(c1, c2)
        self.assertEqual(c3, c4)
        self.assertEqual(c5, c6)
        self.assertEqual(c7, c8)


class Test_Contrast(unittest.TestCase):

    def test_white_white(self):
        '''The Contrast of White and White should be 1'''
        self.assertAlmostEqual(colourettu.contrast('#FFF', '#FFF'), 1)

    def test_black_black(self):
        '''The Contrast of Black and Black should be 1'''
        self.assertAlmostEqual(colourettu.contrast('#000', '#000'), 1)

    def test_white_black(self):
        '''The Contrast of White and Black should be 21'''
        self.assertAlmostEqual(colourettu.contrast('#FFF', '#000'), 21)

    def test_from_colour(self):
        '''The Contrast of a provided colour against White'''
        colour1 = colourettu.Colour("#cde")
        self.assertTrue(colour1.contrast("#FFF"))


class Test_Luminance(unittest.TestCase):

    def test_white_hex(self):
        '''The Luminance of White (as hex) should be 1'''
        self.assertAlmostEqual(colourettu.luminance('#FFFFFF'), 1)

    def test_white_tuple(self):
        '''The Luminance of White (as a tuple) should be 1'''
        self.assertAlmostEqual(colourettu.luminance((255, 255, 255)), 1)

    def test_black(self):
        '''The Luminance of Black should be 0'''
        self.assertAlmostEqual(colourettu.luminance('#000'), 0)

    def test_colour_provided(self):
        '''The Luminance of a provided colour'''
        colour1 = colourettu.Colour()
        self.assertAlmostEqual(colourettu.luminance(colour1), 1)

    def test_as_colour_property(self):
        '''The Luminance of as a property of a colour'''
        colour1 = colourettu.Colour()
        self.assertAlmostEqual(colour1.luminance(), 1)


class Test_Blend(unittest.TestCase):

    def test_white_white(self):
        '''Blending White with White should still be White.'''
        c1 = colourettu.Colour('#fff')
        c2 = colourettu.Colour([255, 255, 255])

        c3 = colourettu.blend(c1, c2)
        self.assertEqual(c3, c1)

    def test_black_black(self):
        '''Blending Black with Black should still be Black.'''
        c1 = colourettu.Colour('#000')
        c2 = colourettu.Colour([0, 0, 0])

        c3 = colourettu.blend(c1, c2)
        self.assertEqual(c3, c1)

    def test_white_black(self):
        '''Blending White with Black should still be dark grey.'''
        c1 = colourettu.Colour('#fff')
        c2 = colourettu.Colour('#000')
        c_expected = colourettu.Colour([180, 180, 180])

        c3 = colourettu.blend(c1, c2)
        self.assertEqual(c3, c_expected)

    def test_black_white(self):
        '''
        Order shouldn't matter. Blending White with Black should still be
        dark grey again.
        '''
        c1 = colourettu.Colour('#fff')
        c2 = colourettu.Colour('#000')
        c_expected = colourettu.Colour([180, 180, 180])

        c3 = colourettu.blend(c2, c1)
        self.assertEqual(c3, c_expected)


def main():
    unittest.main()


if __name__ == '__main__':
    main()



import unittest

import colourettu


class Test_Setup(unittest.TestCase):

    def test_we_live(self):
        '''Test we should *always* pass'''
        pass

    def test_version(self):
        '''Version is available'''
        self.assertIsNotNone(colourettu.__version__)


def main():
    unittest.main()


if __name__ == '__main__':
    main()




import os
from pathlib import Path
import re
import shutil
import subprocess

import colorama
from minchin import text
from invoke import run, task
import semantic_version
from semantic_version import Version
# also requires `twine`

# assumed Invoke configuration file points to Windows Shell

__version__ = "0.2.2"

"""
Changelog:

v0.2.2
======

- move configuration to the top here

"""


p = Path(__file__).parent  # directory holding this file


#@task
def build(ctx):
    pass


#@task
def test(ctx):
    pass


version_re = re.compile(r"__version__ = [\"\']{1,3}(?P<major>\d+)\.(?P<minor>\d+).(?P<patch>\d+)(?:-(?P<prerelease>[0-9A-Za-z\.]+))?(?:\+[0-9A-Za-z-\.]+)?[\"\']{1,3}")
bare_version_re = re.compile(r"__version__ = [\"\']{1,3}([\.\dA-Za-z+-]*)[\"\']{1,3}")
list_match_re = re.compile(r"(?P<leading>[ \t]*)(?P<mark>[-\*+]) +:\w+:")


def here_directory():
    return(Path.cwd())


def module_name():
    return("colourettu")


def source_directory():
    return(here_directory() / 'colourettu')


def test_directory():
    return(source_directory() / 'test')


def doc_directory():
    return(here_directory() / 'docs')


def dist_directory():
    return(here_directory() / 'dist')


def version_file():
    return(source_directory() / '__init__.py')


def changelog_file():
    return(doc_directory() / "changelog.rst")


def server_url(server_name):
    server_name = server_name.lower()
    if server_name in ["testpypi", "pypitest"]:
        return(r"https://testpypi.python.org/pypi")
    elif server_name in ["pypi", ]:
        return(r"https://pypi.python.org/pypi")


def update_version_number(update_level='patch'):
    """Update version number

    Returns a semantic_version object"""

    """Find current version"""
    temp_file = version_file().parent / ("~" + version_file().name)
    with open(str(temp_file), 'w') as g:
        with open(str(version_file()), 'r') as f:
            for line in f:
                version_matches = bare_version_re.match(line)
                if version_matches:
                    bare_version_str = version_matches.groups(0)[0]
                    if semantic_version.validate(bare_version_str):
                        current_version = Version(bare_version_str)
                        print("{}Current version is {}".format(" "*4, current_version))
                    else:
                        current_version = Version.coerce(bare_version_str)
                        if not text.query_yes_quit("{}I think the version is {}. Use it?".format(" "*4, current_version), default="yes"):
                            exit(colorama.Fore.RED + 'Please set an initial version number to continue')

                    """Determine new version number"""
                    if update_level is 'major':
                        current_version = current_version.next_major()
                    elif update_level is 'minor':
                        current_version = current_version.next_minor()
                    elif update_level is 'patch':
                        current_version = current_version.next_patch()
                    elif update_level is 'prerelease':
                        if not current_version.prerelease:
                            current_version = current_version.next_patch()
                            current_version.prerelease = ('dev', )
                    elif update_level is None:
                        # don't update version
                        pass
                    else:
                        exit(colorama.Fore.RED + 'Cannot update version in {} mode'.format(update_level))

                    print("{}New version is     {}".format(" "*4, current_version))

                    """Update version number"""
                    line = '__version__ = "{}"\n'.format(current_version)
                print(line, file=g, end="")
        #print('', file=g)  # add a blank line at the end of the file
    shutil.copyfile(str(temp_file), str(version_file()))
    os.remove(str(temp_file))
    return(current_version)


def build_distribution():
    build_status = subprocess.check_call(['python', 'setup.py', 'sdist', 'bdist_egg', 'bdist_wheel'])
    if build_status is not 0:
        exit(colorama.Fore.RED + 'Something broke tyring to package your code...')


def other_dependancies(server, environment):
    """
    Installs things that need to be in place before installing the main package
    """
    print('  ** Other Dependancides, based on server', server, '**')
    server = server.lower()
    # Pillow is not on TestPyPI
    if server is "local":
        pass
    elif server in ["testpypi", "pypitest"]:
        # these are packages not available on the test server, so install them
        # off the regular pypi server
        print("  **Install Pillow**")
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', 'Pillow'], shell=True)
    elif server in ["pypi"]:
        pass
    else:
        print("  **Nothing more to install**")


def check_local_install(version, ext, server="local"):
    all_files = list(dist_directory().glob('*.{}'.format(ext)))
    the_file = all_files[0]
    for f in all_files[1:]:
        if f.stat().st_mtime > the_file.stat().st_mtime:
            the_file = f

    environment = 'env-{}-{}-{}'.format(version, ext, server)
    if server == "local":
        pass
    else:
        # upload to server
        print("  **Uploading**")
        subprocess.call(['twine', 'upload', str(the_file), '-r', server])

    if (here_directory() / environment).exists():
        shutil.rmtree(environment)  # remove directory if it exists
    subprocess.call(['python', '-m', 'venv', environment])
    if server == "local":
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', str(the_file), '--no-cache'], shell=True)
    else:
        other_dependancies(server, environment)
        print("  **Install from server**")
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', '-i', server_url(server), module_name() + "==" + str(version), '--no-cache'], shell=True)
    print("  **Test version of install package**")
    test_version = subprocess.check_output([environment + '\\Scripts\\python.exe', '-c', "exec(\"\"\"import {0}\\nprint({0}.__version__)\\n\"\"\")".format(module_name())], shell=True)
    test_version = test_version.decode('ascii').strip()
    # print(test_version, type(test_version), type(expected_version))
    if (Version(test_version) == version):
        print('{}{} install {} works!{}'.format(colorama.Fore.GREEN, server, ext, colorama.Style.RESET_ALL))
    else:
        exit('{}{} install {} broken{}'.format(colorama.Fore.RED, server, ext, colorama.Style.RESET_ALL))


@task
def make_release(cts):
    '''Make and upload the release.

    Changelog:
     - v0.2.1 -- 2016-11-18 -- specify downloading of non-cached version of the
                               package for multiple formats can be properly and
                               individually tested.
     - 0.2.2 -- 2016-11028 -- move configuration to top of file
    '''

    make_release_version = __version__
    colorama.init()
    text.title("Minchin 'Make Release' for Python v{}".format(make_release_version))
    print()
    text.subtitle("Configuration")
    print("base dir     -> {}".format(here_directory()))
    print("source       -> .\{}\\".format(source_directory().relative_to(here_directory())))
    print("test dir     -> .\{}\\".format(test_directory().relative_to(here_directory())))
    #print("doc dir      -> .\{}\\".format(doc_directory().relative_to(here_directory())))
    print("version file -> .\{}".format(version_file().relative_to(here_directory())))
    print()
    text.subtitle("Git -- Clean directory?")
    print()
    text.subtitle("Sort Import Statements")
    print()
    text.subtitle("Run Tests")
    print()
    text.subtitle("Update Version Number")
    new_version = update_version_number(None)
    print()
    text.subtitle("Add Release to Changelog")
    print()
    text.subtitle("Build Documentation")
    print()
    text.query_yes_quit('All good and ready to go?')

    text.subtitle("Build Distributions")
    build_distribution()

    for server in [
                    #"local",
                    "testpypi",
                    "pypi",
                  ]:
        for file_format in ["tar.gz", "whl"]:
            print()
            text.subtitle("Test {} Build {}".format(file_format, server))
            check_local_install(new_version, file_format, server)

    # new_version = update_version_number('prerelease')



'''Eventaully, this file will execute the necessary steps.
For now, it just prints what they are.'''

import os
import re
import shutil
import subprocess
from pathlib import Path
from sys import exit

import colorama
import semantic_version
from semantic_version import Version
from colorama import Fore, Style
from isort import SortImports
from minchin import text
from datetime import datetime

# also requires:
# - green
# - pip
# - setuptools
# - wheel
#
# assumes:
# - git on commandline
# - tests written for green
# - green available on command line
# - a valid `.pypirc` file with credentials for TestPyPI and PyPI
# - your project is already registered on TestPyPi and PyPI

# version of this script
__version__ = "0.1.0"

version_re = re.compile(r"__version__ = [\"\']{1,3}(?P<major>\d+)\.(?P<minor>\d+).(?P<patch>\d+)(?:-(?P<prerelease>[0-9A-Za-z\.]+))?(?:\+[0-9A-Za-z-\.]+)?[\"\']{1,3}")
bare_version_re = re.compile(r"__version__ = [\"\']{1,3}([\.\dA-Za-z+-]*)[\"\']{1,3}")
list_match_re = re.compile(r"(?P<leading>[ \t]*)(?P<mark>[-\*+]) +:\w+:")


def here_directory():
    return(Path.cwd())


def module_name():
    return("colourettu")


def source_directory():
    return(here_directory() / module_name())


def test_directory():
    return(here_directory() / 'tests')


def doc_directory():
    return(here_directory() / 'docs')


def dist_directory():
    return(here_directory() / 'dist')


def version_file():
    return(source_directory() / "__init__.py")


def changelog_file():
    return(here_directory() / 'docs' / "changelog.rst")


def other_dependancies(server, environment):
    """Installs things that need to be in place before installing the main package"""
    server = server.lower()
    # Pillow is not on TestPyPI
    if server is "local":
        pass
    elif server in ["testpypi", "pypitest"]:
        print("  **Install Pillow**")
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', 'Pillow'], shell=True)
    elif server is "pypi":
        pass
    else:
        print("  **Nothing more to install**")


def server_url(server_name):
    server_name = server_name.lower()
    if server_name in ["testpypi", "pypitest"]:
        return(r"https://testpypi.python.org/pypi")
    elif server_name in ["pypi", ]:
        return(r"https://pypi.python.org/pypi")
    else:
        # we really should throw an error here...
        return()

# //-------------------------------------------------------------------------//


def git_check():
    """Check for uncomitted changes"""
    git_status = subprocess.check_output(['git', 'status', '--porcelain'])
    if len(git_status) is 0:
        print(Fore.GREEN + 'All changes committed' + Style.RESET_ALL)
    else:
        exit(Fore.RED + 'Please commit all files to continue')


def sort_imports(my_dir):
    """Sort Imports

    Assumes `my_dir` is a Pathlib object"""

    for f in my_dir.glob('**/*.py'):
        SortImports(str(f))


def run_tests():
    """Run tests"""
    test_status = subprocess.check_call(['green', str(test_directory()), '-vv'])
    if test_status is not 0:
        exit(Fore.RED + 'Please make all tests pass to continue')


def update_version_number(update_level='patch'):
    """Update version number

    Returns a semantic_version object"""

    """Find current version"""
    temp_file = version_file().parent / ("~" + version_file().name)
    with open(str(temp_file), 'w') as g:
        with open(str(version_file()), 'r') as f:
            for line in f:
                version_matches = bare_version_re.match(line)
                if version_matches:
                    bare_version_str = version_matches.groups(0)[0]
                    if semantic_version.validate(bare_version_str):
                        current_version = Version(bare_version_str)
                        print("{}Current version is {}".format(" "*4, current_version))
                    else:
                        current_version = Version.coerce(bare_version_str)
                        if not text.query_yes_quit("{}I think the version is {}. Use it?".format(" "*4, current_version), default="yes"):
                            exit(Fore.RED + 'Please set an initial version number to continue')

                    """Determine new version number"""
                    if update_level is 'major':
                        current_version = current_version.next_major()
                    elif update_level is 'minor':
                        current_version = current_version.next_minor()
                    elif update_level is 'patch':
                        current_version = current_version.next_patch()
                    elif update_level is 'prerelease':
                        if not current_version.prerelease:
                            current_version = current_version.next_patch()
                            current_version.prerelease = ('dev', )
                    else:
                        exit(Fore.RED + 'Cannot update version in {} mode'.format(update_level))

                    print("{}New version is     {}".format(" "*4, current_version))

                    """Update version number"""
                    line = '__version__ = "{}"'.format(current_version)
                print(line, file=g, end="")
        print('', file=g)  # add a blank line at the end of the file
    shutil.copyfile(str(temp_file), str(version_file()))
    os.remove(str(temp_file))
    return(current_version)


def add_release_to_changelog(version):
    """Add release line at the top of the first list it finds

    Assumes your changelog in managed with `releases`"""
    temp_file = changelog_file().parent / ("~" + changelog_file().name)
    now = datetime.today()
    release_added = False
    with open(str(temp_file), 'w') as g:
        with open(str(changelog_file()), 'r') as f:
            for line in f:
                list_match = list_match_re.match(line)
                if list_match and not release_added:
                    release_line = "{}{} :release:`{} <{}-{:02}-{:02}>`".format(
                                        list_match.group("leading"),
                                        list_match.group("mark"),
                                        version, now.year, now.month, now.day)
                    print(release_line, file=g)
                    release_added = True
                print(line, file=g, end="")
            if not release_added:
                release_line = "{}{} :release:`{} <{}-{:02}-{:02}>`".format(
                                " ", "-", version, now.year, now.month, now.day)
                print(release_line, file=g)
        print('', file=g)  # add a blank line at the end of the file
    shutil.copyfile(str(temp_file), str(changelog_file()))
    os.remove(str(temp_file))


def run_sphinx():
    """Runs Sphinx via it's `make html` command"""
    old_dir = here_directory()
    os.chdir(str(doc_directory()))
    doc_status = subprocess.check_call(['make', 'html'], shell=True)
    os.chdir(str(old_dir))  # go back to former working directory
    if doc_status is not 0:
        exit(Fore.RED + 'Something broke generating your documentation...')


def build_distribution():
    build_status = subprocess.check_call(['python', 'setup.py', 'sdist', 'bdist_egg', 'bdist_wheel'])
    if build_status is not 0:
        exit(Fore.RED + 'Something broke tyring to package your code...')


def check_local_install(version, ext, server="local"):
    all_files = list(dist_directory().glob('*.{}'.format(ext)))
    the_file = all_files[0]
    for f in all_files[1:]:
        if f.stat().st_mtime > the_file.stat().st_mtime:
            the_file = f

    environment = 'env-{}-{}-{}'.format(version, ext, server)
    if server == "local":
        pass
    else:
        # upload to server
        print("  **Uploading**")
        subprocess.call(['twine', 'upload', str(the_file), '-r', server])

    if (here_directory() / environment).exists():
        shutil.rmtree(environment)  # remove directory if it exists
    subprocess.call(['virtualenv', environment])
    if server == "local":
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', str(the_file)], shell=True)
    else:
        other_dependancies(server, environment)
        print("  **Install from server**")
        subprocess.call([environment + '\\Scripts\\pip.exe', 'install', '-i', server_url(server), module_name() + "==" + str(version)], shell=True)
    print("  **Test version of install package**")
    test_version = subprocess.check_output([environment + '\\Scripts\\python.exe', '-c', "exec(\"\"\"import {0}\\nprint({0}.__version__)\\n\"\"\")".format(module_name())], shell=True)
    test_version = test_version.decode('ascii').strip()
    # print(test_version, type(test_version), type(expected_version))
    if (Version(test_version) == version):
        print('{}{} install {} works!{}'.format(Fore.GREEN, server, ext, Style.RESET_ALL))
    else:
        exit('{}{} install {} broken{}'.format(Fore.RED, server, ext, Style.RESET_ALL))


def print_all_steps():
    # see https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
    print('Tag release')
    print('   tag v(version number)')
    print('   push tag')
    print('Push new documentation')
    print('    cd ..\colourettu-gh-pages')
    print('   (copy changes)')
    print('   git add -a')
    print('   git commit -m "Documentation updated!"')
    print('   cd ..\colourettu')
    print('Push to git repo, including tags')

    # swtich to twine  https://pypi.python.org/pypi/twine/
    # twine upload dist/*

# //-------------------------------------------------------------------------//


def main():
    colorama.init()
    text.title("Minchin 'Make Release' for Python v{}".format(__version__))
    print()
    text.subtitle("Configuration")
    print("base dir     -> {}".format(here_directory()))
    print("source       -> .\{}\\".format(source_directory().relative_to(here_directory())))
    print("test dir     -> .\{}\\".format(test_directory().relative_to(here_directory())))
    print("doc dir      -> .\{}\\".format(doc_directory().relative_to(here_directory())))
    print("version file -> .\{}".format(version_file().relative_to(here_directory())))
    print()

    print()
    text.subtitle("Git -- Clean directory?")
    # git_check()

    print()
    text.subtitle("Sort Import Statements")
    # sort_imports(source_directory())
    # sort_imports(test_directory())

    print()
    text.subtitle("Run Tests")
    # run_tests()

    print()
    text.subtitle("Update Version Number")
    # new_version = update_version_number('patch')
    new_version = update_version_number('prerelease')

    print()
    text.subtitle("Add Release to Changelog")
    # add_release_to_changelog(new_version)

    print()
    text.subtitle("Build Documentation")
    # run_sphinx()

    print()
    text.subtitle("Build Distributions")
    build_distribution()

    # for server in ["local", "testpypi", "pypi"]:
    for server in ["testpypi"]:
        for file_format in ["zip", "whl"]:
            print()
            text.subtitle("Test {} Build {}".format(file_format, server))
            check_local_install(new_version, file_format, server)
    # tests will pass if the local version of colourettu can be imported
    # source files should be moved to a 'src' directory, but this will require
    #    the module to be properly installed to run tests, generate docs, etc.
    #
    # uploads -- you cannot upload a file with the same name. Consider adding
    #    build information to version number to allow retries updating...
    #
    # registering -- does not attempt

    # after everything is done
    # update_version_number('prerelease')

if __name__ == '__main__':
    main()



import os
import unittest
from unittest import skip

import colourettu


class Test_Palette(unittest.TestCase):

    def test_repr(self):
        '''Representation of `palette` class'''
        p1 = colourettu.Palette()
        self.assertEqual(p1.__repr__(), "<colourettu.Palette #FFFFFF to #000000, 2 colours>")

    def test_str(self):
        '''String representation of `palette` class'''
        p1 = colourettu.Palette()
        self.assertEqual(p1.__str__(), "#FFFFFF, #000000")

    def test_length(self):
        '''length function on a of `palette` class'''
        p1 = colourettu.Palette()
        self.assertEqual(len(p1), 2)

    def test_from_list_list(self):
        my_list = [colourettu.Colour('#aaa'), colourettu.Colour('#123'), colourettu.Colour('#345682')]
        p1 = colourettu.Palette()
        p1.from_list(my_list)
        self.assertEqual(len(p1), 3)
        self.assertEqual(p1._start, colourettu.Colour('#aaaaaa'))
        self.assertEqual(p1._end, colourettu.Colour('#345682'))

    def test_from_list_tuple(self):
        my_tuple = (colourettu.Colour('#111'), colourettu.Colour('#983'), colourettu.Colour('#aabcde'))
        p1 = colourettu.Palette()
        p1.from_list(my_tuple)
        self.assertEqual(len(p1), 3)
        self.assertEqual(p1._start, colourettu.Colour('#111111'))
        self.assertEqual(p1._end, colourettu.Colour('#aabcde'))

    def test_convert_start_colour(self):
        p1 = colourettu.Palette(start_colour='#124')
        self.assertEqual(p1._start, colourettu.Colour('#112244'))

    def test_convert_end_colour(self):
        p1 = colourettu.Palette(end_colour='#a7f')
        self.assertEqual(p1._end, colourettu.Colour('#aa77ff'))

    def test_invalid_start_colour(self):
        with self.assertRaises(ValueError):
            p1 = colourettu.Palette(start_colour="more")

    def test_invalid_end_colour(self):
        with self.assertRaises(ValueError):
            p1 = colourettu.Palette(end_colour=[1, 2, 3, 4])

    def test_from_list_convert_list_colour(self):
        my_list = [[194, 229, 174],
                   [146, 149, 192]]

        p1 = colourettu.Palette()
        p1.from_list(my_list)
        self.assertEqual(len(p1), 2)
        self.assertEqual(p1._start, colourettu.Colour('#c2e5ae'))
        self.assertEqual(p1._end, colourettu.Colour('#9295c0'))

    def test_palette_plus_colour(self):
        p1 = colourettu.Palette()
        c2 = colourettu.Colour('#123')
        p2 = p1 + c2
        self.assertEqual(len(p2), 3)
        self.assertEqual(p2._start, colourettu.Colour('#fff'))
        self.assertEqual(p2._end, colourettu.Colour('#112233'))

    def test_colour_plus_palette(self):
        p1 = colourettu.Palette()
        c2 = colourettu.Colour('#abc')
        p2 = c2 + p1
        self.assertEqual(len(p2), 3)
        self.assertEqual(p2._start, colourettu.Colour('#aabbcc'))
        self.assertEqual(p2._end, colourettu.Colour('#000'))

    def test_palette_plus_palette(self):
        p1 = colourettu.Palette()
        p2 = colourettu.Palette('#abc', '#345')
        p3 = p1 + p2
        self.assertEqual(len(p3), 4)
        self.assertEqual(p3._start, colourettu.Colour('#fff'))
        self.assertEqual(p3._end, colourettu.Colour('#345'))

    def test_palette_plus_palette_overlapping_colour(self):
        p1 = colourettu.Palette('#123', '#456')
        p2 = colourettu.Palette('#456', '#789')
        p3 = p1 + p2
        self.assertEqual(len(p3), 3)
        self.assertEqual(p3._start, colourettu.Colour('#123'))
        self.assertEqual(p3._end, colourettu.Colour('#789'))
        self.assertEqual(p3._colours[1], colourettu.Colour('#456'))

    def test_blend_black_white(self):
        p1 = colourettu.Palette()  # black, white palette is default
        p1.blend()

        self.assertEqual(len(p1), 3)
        self.assertEqual(p1._start, colourettu.Colour('#fff'))
        self.assertEqual(p1._end, colourettu.Colour('#000'))
        self.assertEqual(p1._colours[1], colourettu.Colour([180, 180, 180]))

    def test_blend_multicycles(self):
        p1 = colourettu.Palette()  # black, white palette is default
        p1.blend(3)

        self.assertEqual(len(p1), 9)
        self.assertEqual(p1._start, colourettu.Colour('#fff'))
        self.assertEqual(p1._end, colourettu.Colour('#000'))
        self.assertEqual(p1._colours[4], colourettu.Colour([180, 180, 180]))


class Test_Palette_with_Images(unittest.TestCase):

    def tearDown(self):
        try:
            os.remove('palette.png')
        except:
            pass

    def test_to_image(self):
        p1 = colourettu.Palette()
        p1.to_image()
        self.assertTrue(os.path.isfile('palette.png'))

    @skip('not yet defined')
    def test_to_image_alpha_channel(self):
        pass

    @skip('not yet defined')
    def test_to_image_band_width(self):
        pass

    @skip('not yet defined')
    def test_to_image_max_width(self):
        pass


def main():
    unittest.main()


if __name__ == '__main__':
    main()



# This is part of colourettu. See http://minchin.ca/colourettu/

import math


class Colour:
    """
    Base class for dealing with colours.

    Args:
        mycolour (str, list, or tuple):  string of hex representation of colour
            you are creating, or
            a 3 item list or tuple of the red, green, and blue channels of
            the colour you are creating. Default is "#FFF" (white).
        normalized_rgb (bool): whether the values for the red, green, and blue
            channels are *normalized* (i.e. values scaled from 0 to 1) or not
            (i.e. values scaled from 0 to 255). Default is *False*.

    Colours are created by calling the ``Colour`` class. Colour values can
    be provided via 3 or 6 digit hex notation, or providing a list or a
    tuple of the Red, Green, and Blue values (as integers, if
    ``normalized_rgb=False``, or as floating numbers between 0 and 1 if
    ``normalized_rgb=True``).

    .. code:: python

        import colourettu
        from colourettu import Colour

        c1 = Colour()        # defaults to #FFF
        c2 = Colour("#eee")  # equivalent to #EEEEEE
        c3 = Colour("#456bda")
        c4 = Colour([3, 56, 129])
        c5 = Colour((63, 199, 233))
        c6 = Colour([0.242, 0.434, 0.165], normalized_rgb=True)

    The value of each channel can be pulled out:

    .. code:: pycon

        >>> c4.red()
        3
        >>> c4.green()
        56
        >>> c4.blue()
        129

    You can also get the colour back as either a hex value, or a rgb tuple:

    .. code:: pycon

        >>> c2.hex()
        '#EEEEEE'
        >>> c2.rgb()
        (238, 238, 238)

    Colours are considered equal is the values of the R, G, and B channels
    match.

    .. code:: pycon

        >>> c1 == c2
        False
        >>> c2 == Color([238, 238, 238])
        True
    """

    _r = _g = _b = None

    def __init__(self, mycolour="#FFF", normalized_rgb=False):
        if type(normalized_rgb) is not bool:
            raise TypeError('normalized_rgb must be either True or False')

        if type(mycolour) is str:
            if mycolour.startswith("#"):
                myhex = mycolour[1:]
                if len(myhex) % 3 != 0:
                    raise ValueError("Invalid Hex Colour")
                thirds = int(len(myhex)/3)
                r, g, b = myhex[0:thirds], myhex[thirds:2*thirds], \
                    myhex[2*thirds:3*thirds]
                if len(r) == 1:
                    r = r + r
                if len(g) == 1:
                    g = g + g
                if len(b) == 1:
                    b = b + b
                self._r = int(r, 16)
                self._g = int(g, 16)
                self._b = int(b, 16)
            else:
                raise ValueError("Strings must start with '#'")
        elif type(mycolour) in (list, tuple):
            if len(mycolour) == 3:
                if not normalized_rgb:
                    if((type(mycolour[0]) is int)
                            and (type(mycolour[1]) is int)
                            and (type(mycolour[2]) is int)):
                        self._r, self._g, self._b = mycolour
                    else:
                        raise TypeError('Tuple and Lists must be three'
                                        'integers if normalized_rgb=False.')
                else:
                    if ((type(mycolour[0]) in (float, int)) and
                            (type(mycolour[1]) in (float, int)) and
                            (type(mycolour[2]) in (float, int))):
                        if((0 <= mycolour[0] <= 1) and
                           (0 <= mycolour[1] <= 1) and
                           (0 <= mycolour[2] <= 1)):
                            self._r = int(mycolour[0]*255)
                            self._g = int(mycolour[1]*255)
                            self._b = int(mycolour[2]*255)
                        else:
                            raise ValueError('Normalized RGB values must be'
                                             'between 0 and 1.')
                    else:
                        raise TypeError('Tuples and Lists must be three'
                                        'floating point numbers if'
                                        'normalized_rgb=True')
            else:
                raise ValueError('Tuples and Lists must be three items long.')
        else:
            raise TypeError('Must supply a string, a list, or a tuple')

    def __repr__(self):
        return('<colourettu.Colour {}>'.format(self.hex()))

    def __str__(self):
        return('{}'.format(self.hex()))

    def __eq__(self, other):
        """
        Determine if Colours are 'equal'.

        Colours are considered equal if the values of the R, G, and B channels
        match.
        """
        return ((self._r is other.red())
                and (self._g is other.green())
                and (self._b is other.blue()))

    def hex(self):
        """
        Returns the HTML-style hex code for the Colour.

        Returns:
            str: the colour as a HTML-sytle hex string
        """
        return "#{:02x}{:02x}{:02x}".format(self._r, self._g, self._b).upper()

    def red(self):
        """
        Returns the value of the red channel of the Colour.

        Returns:
            int: value of the red channel of the colour
        """
        return self._r

    def green(self):
        """
        Returns the value of the green channel of the Colour.

        Returns:
            int: value of the green channel of the colour
        """
        return self._g

    def blue(self):
        """
        Returns the value of the blue channel of the Colour.

        Returns:
            int: value of the blue channel of the colour
        """
        return self._b

    def rgb(self):
        """
        Returns a tuples of the values of the red, green, and blue channels of
        the Colour.

        Returns:
            tuple: the rgb values of the colour (with values between 0 and 255)
        """
        return(self._r, self._g, self._b)

    def normalized_rgb(self):
        r"""
        Returns a tuples of the normalized values of the red, green, and blue
        channels of the Colour.

        Returns:
            tuple: the rgb values of the colour (with values normalized between
                   0.0 and 1.0)

        .. note::

            Uses the formula:

            \\[ r_{norm} = \\begin{cases}
            \\frac{r_{255}}{12.92}\\ \\qquad &\\text{if $r_{255}$ $\\le$ 0.03928}
            \\\\
            \\left(\\frac{r_{255} + 0.055}{1.055}\\right)^{2.4}
            \\quad &\\text{otherwise}
            \\end{cases} \\]

        `Source <http://www.w3.org/TR/2008/REC-WCAG20-20081211/#relativeluminancedef>`_
        """

        r1 = self._r / 255
        g1 = self._g / 255
        b1 = self._b / 255

        if r1 <= 0.03928:
            r2 = r1 / 12.92
        else:
            r2 = math.pow(((r1 + 0.055) / 1.055), 2.4)
        if g1 <= 0.03928:
            g2 = g1 / 12.92
        else:
            g2 = math.pow(((g1 + 0.055) / 1.055), 2.4)
        if b1 <= 0.03928:
            b2 = b1 / 12.92
        else:
            b2 = math.pow(((b1 + 0.055) / 1.055), 2.4)

        return (r2, g2, b2)

    def luminance(self):
        """Calls the :py:func:`luminance` on the colour defined."""
        return luminance(self)

    def contrast(self, myothercolour):
        """Calls the :py:func:`contrast` on the colour defined."""
        return contrast(self, myothercolour)


def luminance(mycolour):
    r"""Determine (relative) luminance of a colour.

    Args:
        mycolour(colourettu.Colour): a colour

    Luminance is a measure of how 'bright' a colour is. Values are
    normalized so that the Luminance of White is 1 and the Luminance of
    Black is 0. That is to say:

    .. code:: pycon

        >>> colourettu.luminance("#FFF")    # white
        0.9999999999999999
        >>> colourettu.luminance("#000")    # black
        0.0

    ``luminance()`` can also be called on an already existing colour:

    .. code:: pycon

        >>> c3.luminance()
        0.2641668488934239
        >>> colourettu.luminance(c4)
        0.08007571268096524

    .. note::

        Uses the formula:

        \\[ lum = \\sqrt{0.299 r^2 + 0.587 g^2 + 0.114 b^2} \\]
    """

    colour_for_type = Colour()
    if type(mycolour) is type(colour_for_type):
        mycolour2 = mycolour
    else:
        try:
            mycolour2 = Colour(mycolour)
        except:
            raise TypeError("Must supply a colourettu.Colour")

    (r1, g1, b1) = mycolour2.normalized_rgb()

    return math.sqrt(0.299*math.pow(r1, 2) +
                     0.587*math.pow(g1, 2) +
                     0.114*math.pow(b1, 2))


def contrast(colour1, colour2):
    r"""Determines the contrast between two colours.

    Args:
        colour1 (colourettu.Colour): a colour
        colour2 (colourettu.Colour): a second colour

    Contrast the difference in (perceived) brightness between colours.
    Values vary between 1:1 (a given colour on itself) and 21:1 (white on
    black).

    To compute contrast, two colours are required.

    .. code:: pycon

        >>> colourettu.contrast("#FFF", "#FFF") # white on white
        1.0
        >>> colourettu.contrast(c1, "#000") # black on white
        20.999999999999996
        >>> colourettu.contrast(c4, c5)
        4.363552233203198

    ``contrast`` can also be called on an already existing colour, but a
    second colour needs to be provided:

    .. code:: pycon

        >>> c4.contrast(c5)
        4.363552233203198

    .. note::

        Uses the formula:

        \\[ contrast = \\frac{lum_1 + 0.05}{lum_2 + 0.05} \\]

    **Use of Contrast**

    For Basic readability, the ANSI standard is a contrast of 3:1 between
    the text and it's background. The W3C proposes this as a minimum
    accessibility standard for regular text under 18pt and bold text under
    14pt. This is referred to as the *A* standard. The W3C defines a higher
    *AA* standard with a minimum contrast of 4.5:1. This is approximately
    equivalent to 20/40 vision, and is common for those over 80. The W3C
    define an even higher *AAA* standard with a 7:1 minimum contrast. This
    would be equivalent to 20/80 vision. Generally, it is assumed that those
    with vision beyond this would access the web with the use of assistive
    technologies.

    If needed, these constants are stored in the library.

    .. code:: pycon

        >>> colourettu.A_contrast
        3.0
        >>> colourettu.AA_contrast
        4.5
        >>> colourettu.AAA_contrast
        7.0

    I've also found mention that if the contrast is *too* great, this can
    also cause readability problems when reading longer passages. This is
    confirmed by personal experience, but I have been (yet) unable to find
    any quantitative research to this effect.
    """

    colour_for_type = Colour()
    if type(colour1) is type(colour_for_type):
        mycolour1 = colour1
    else:
        try:
            mycolour1 = Colour(colour1)
        except:
            raise TypeError("colour1 must be a colourettu.colour")

    if type(colour2) is type(colour_for_type):
        mycolour2 = colour2
    else:
        try:
            mycolour2 = Colour(colour2)
        except:
            raise TypeError("colour2 must be a colourettu.colour")

    lum1 = mycolour1.luminance()
    lum2 = mycolour2.luminance()

    minlum = min(lum1, lum2)
    maxlum = max(lum1, lum2)

    return (maxlum + 0.05) / (minlum + 0.05)


def blend(colour1, colour2):
    r"""Takes two :py:class:`Colour` s and returns the 'average' Colour.

    Args:
        colour1 (colourettu.Colour): a colour
        colour2 (colourettu.Colour): a second colour

    .. note::

        Uses the formula:

        \\[ r_{blended} = \\sqrt \\frac{r_1^2 + r_2^2}{2} \\]


        It is hown here for the red channel, but applied independantly to each
        of the red, green, and blue channels. The reason for doing it this way
        (rather than using a simple average) is that the brightness of the
        colours is stored in a logrythmic scale, rather than a linear one.

        For a fuller explaination, Minute Physics has released a great
        `YouTube video <https://youtu.be/LKnqECcg6Gw>`_.

    .. seealso:: :py:func:`Palette.blend`
    """
    # raw docstring is needed so that MathJax will render in generated
    # documentation

    # gamma is the power we're going to use to do this conversion
    gamma = 2.0

    # start by normalizing values
    r_1 = colour1.red()/255.
    g_1 = colour1.green()/255.
    b_1 = colour1.blue()/255.
    r_2 = colour2.red()/255.
    g_2 = colour2.green()/255.
    b_2 = colour2.blue()/255.

    r_m = math.pow(((math.pow(r_1, gamma) + math.pow(r_2, gamma)) / 2), 1/gamma)
    g_m = math.pow(((math.pow(g_1, gamma) + math.pow(g_2, gamma)) / 2), 1/gamma)
    b_m = math.pow(((math.pow(b_1, gamma) + math.pow(b_2, gamma)) / 2), 1/gamma)

    c_m = Colour([r_m, g_m, b_m], normalized_rgb=True)

    return c_m


A_contrast = 3.0
AA_contrast = 4.5
AAA_contrast = 7.0
